@@ -1,17 +1,23 @@
 import re
+import regex
 from .config import math_code
 
 match_code = r"(" + math_code + r"_\d+(?:_\d+)*)"
 match_code_replace = math_code + r"_(\d+(?:_\d+)*)*"
+pattern_env = r"\\begin\{(.*?)\}(.*?)\\end\{\1\}"  # \begin{xxx} \end{xxx}, group 1: name, group 2: content
+pattern_command_full = r"\\([a-zA-Z]+\*?)?(\[[a-zA-Z\s,]*?\])?(\{((?:[^{}]++|(?3))++)\})"   # \xxx[xxx]{xxx} and \xxx{xxx}, group 1: name, group 2: option, group 4: content
+pattern_command_simple = r"\\([a-zA-Z]+)"  # \xxx, group 1: name
 
 
 def variable_code(count):
+    # If count is 123, the code is {math_code}_1_2_3
     digits = list(str(count))
     count_str = "_".join(digits)
     return f'{math_code}_{count_str}'
 
 
 def modify_text(text, modify_func):
+    # modify text without touching the variable codes
     split_text = [s for s in re.split(match_code, text) if s is not None]
     for i in range(len(split_text)):
         if not re.match(match_code, split_text[i]):
@@ -21,12 +27,15 @@ def modify_text(text, modify_func):
 
 
 def modify_before(text):
+    # mathpix is stupid so sometimes does not add $ $ for \pm
     text = text.replace('\\pm', '$\\pm$')
+    # the "." may be treated as end of sentence
     text = text.replace('Eq.', 'equation')
     return text
 
 
 def modify_after(text):
+    # the "_" in the text should be replaced to "\_"
     pattern = r"(?<!\\)_"
     text = re.sub(pattern, r"\\_", text)
     return text
@@ -41,27 +50,23 @@ def replace_latex_envs(text):
     $ $, \( xxx \), \xxx[xxx]{xxx}, \xxx{xxx}, and \xxx.
     Returns the processed text and a list of replaced LaTeX environments.
     """
-    # TODO: for \xxx[xxx]{xxx} and \xxx{xxx} the regex here cannot process \xxx{xxx{xxx}} correctly.
-    # We need to either change the regex or use the function "process_specific_env" in the following.
-    # Here is a regex that works better but cannot process \xxx{xxx \{xxx\}}: r'(?<!\\)\\[a-zA-Z]+?(\{(?:[^{}]++|(?1))++\})'
 
     # define regular expressions for each LaTeX environment
     latex_env_regex = [
-        r"(?<!\\)\$\$(.*?)(?<!\\)\$\$",  # $$ $$
-        r"(?<!\\)\$(.*?)(?<!\\)\$",  # $ $
-        r"(?<!\\)\\\[(.*?)(?<!\\)\\\]",  # \[ xxx \]
-        r"(?<!\\)\\\((.*?)(?<!\\)\\\)",  # \( xxx \)
-        r"(?<!\\)\\begin\{(.*?)\}(.*?)(?<!\\)\\end\{\1\}",  # \begin{xxx} \end{xxx}
-        r"(?<!\\)\\([a-zA-Z]+)\*?\[(.*?)\]\{(.*?)\}",  # \xxx[xxx]{xxx}
-        r"(?<!\\)\\([a-zA-Z]+)\*?\{(.*?)\}",  # \xxx{xxx}
-        r"(?<!\\)\\([a-zA-Z]+)",  # \xxx
+        r"\$\$(.*?)\$\$",  # $$ $$
+        r"\$(.*?)\$",  # $ $
+        r"\\\[(.*?)\\\]",  # \[ xxx \]
+        r"\\\((.*?)\\\)",  # \( xxx \)
+        pattern_env,  # \begin{xxx} \end{xxx}
+        pattern_command_full,  # \xxx[xxx]{xxx}
+        pattern_command_simple,  # \xxx
     ]
 
     # iterate through each LaTeX environment and replace with "XMATH_{digit1}_{digit2}_..._{digit_last}"
     count = 0
     replaced_envs = []
-    for regex in latex_env_regex:
-        pattern = re.compile(regex, re.DOTALL)
+    for regex_symbol in latex_env_regex:
+        pattern = regex.compile(regex_symbol, regex.DOTALL)
         while pattern.search(text):
             latex_env = pattern.search(text).group()
             replaced_envs.append(f' {latex_env} ')
@@ -72,7 +77,7 @@ def replace_latex_envs(text):
     return text, replaced_envs
 
 
-def recover_latex_envs(text, replaced_envs):
+def recover_latex_envs(text, replaced_envs, verbose=False):
     nenvs = len(replaced_envs)
     matched_indices = []
 
@@ -83,6 +88,7 @@ def recover_latex_envs(text, replaced_envs):
             return replaced_envs[index]
         else:
             return '???'
+
     text = modify_text(text, modify_after)
     pattern = re.compile(match_code_replace)
     total_num = 0
@@ -95,8 +101,8 @@ def recover_latex_envs(text, replaced_envs):
     n_bad1 = len(matched_indices) - n_good
     n_bad2 = nenvs - n_good
     n_bad = max(n_bad1, n_bad2)
-    if n_bad > 0:
-        print(n_bad, 'latex environments are wrong in total', nenvs)
+    if verbose and n_bad > 0:
+        print(n_bad, 'latex environments are probably wrong in total', nenvs)
     return text
 
 
@@ -104,13 +110,11 @@ def remove_tex_comments(text):
     """
     Removes all TeX comments in a given string with the format "% comment text".
     Does not match "\%".
+    If "%" is at the beginning of a line then delete this line.
     Returns the processed string.
     """
-    # define regular expression for TeX comments
-    comment_regex = r"(?<!\\)%.*?(?=$|\n)"
-
-    # remove comments from text
-    text = re.sub(comment_regex, "", text)
+    text = re.sub(r"\n(?<!\\)%.*?(?=\n)", "", text)
+    text = re.sub(r"(?<!\\)%.*?(?=\n)", "", text)
 
     return text
 
@@ -136,27 +140,37 @@ def count_braces(text):
     return text.count('{'), text.count('}')
 
 
-def process_specific_env(text, pattern_begin, pattern_end, function):
-    position = 0
-    while True:
-        position = text.find(pattern_begin, position)
-        if position == -1:
-            break
-        position += len(pattern_begin)
-        start = position
-        while True:
-            position = text.find(pattern_end, position)
-            if position > 0 and text[position - 1] != '\\':
-                n_left, n_right = count_braces(text[start:position])
-                if n_left == n_right:
-                    break
-            position += 1
-        text_before = text[0:start]
-        text_middle = function(text[start:position])
-        text_after = text[position:]
-        position = len(text_before) + len(text_middle) + len(pattern_end)
-        text = text_before + text_middle + text_after
-    return text
+def process_specific_env(latex, function, env_names):
+    pattern = regex.compile(pattern_env, regex.DOTALL)
+
+    def process_function(match):
+        # \begin{env_name} content \end{env_name}
+        env_name = match.group(1)
+        content = match.group(2)
+        if env_name in env_names:
+            processed_content = function(content)
+            return rf'\begin{{{env_name}}}{processed_content}\end{{{env_name}}}'
+        else:
+            return match.group(0)
+    return pattern.sub(process_function, latex)
+
+
+def process_specific_commands(latex, function, command_names):
+    pattern = regex.compile(pattern_command_full, regex.DOTALL)
+
+    def process_function(match):
+        # \{command_name}[options]{content}
+        command_name = match.group(1)
+        options = match.group(2)
+        if options is None:
+            options = ''
+        content = match.group(4)
+        if command_name in command_names:
+            processed_content = function(content)
+            return rf'\{command_name}{options}{{{processed_content}}}'
+        else:
+            return match.group(0)
+    return pattern.sub(process_function, latex)
 
 
 def remove_blank_lines(text):
